/**
Test for EPGPU OpenCL library: simple 2D rendering

Dr. Orion Lawlor, lawlor@alaska.edu, 2011-04-27 (Public Domain)
*/
#include <iostream>
#include <stdlib.h>
#include "epgpu.h" /* Orion Lawlor's Expressive Programming for GPU library */
#include "epgpu.cpp" /* <- just compile it here, now.  No compiler/DLL hassles! */

GPU_STRUCT(vec2, // <- struct's name
// OpenCL and C++ declaration of fields
	float x; 
	float y;
,
// C++-only methods
	vec2() {}
	vec2(float x_,float y_) { x=x_; y=y_; }
	vec2 &operator+=(const vec2 &rhs) {x+=rhs.x; y+=rhs.y; return *this; }
	friend vec2 operator+(vec2 lhs,const vec2 &rhs) {lhs+=rhs; return lhs;}
)

GPU_STRUCT(array2d_info, // <- struct's name
// OpenCL and C++ declaration of fields
	__global<float *> base; // base address (cl_mem pointer)
	int w; // width (i or x size)
	int h; // height (j or y size)
,
// C++-only methods (empty)
	array2d_info() {base=0; w=h=0; }
	array2d_info(__global<float *> base_,int w_,int h_)
		{ base=base_; w=w_; h=h_; }
)

GPU_ADD_STATIC_CODE(
"#pragma OPENCL EXTENSION cl_khr_global_int32_base_atomics : enable\n"
,extension_atomic_inc_pragma);

int main(int argc,char *argv[]) {
	int w,h; w=h=10;
	if (argc>2) {w=atoi(argv[1]); h=atoi(argv[2]); }
	
	static GPU_KERNEL_2D(writeArray, (int w, __global<vec2 *> dest,__global<int *> count),
		dest[i+w*j].x=100000*j+1000*i+3;
		if (i>w/2) atom_inc(count);
	)
	static GPU_FILLKERNEL_2D(float, fillArray, (), 
		result=17+i;
	)
	
	std::cout<<"Generated code: "<<gpu_env::static_env().show_code(1);
	
	gpu_vector<int> count(1); 
	static GPU_FILLKERNEL(int, zeroInt, (), result=0; )
	count=zeroInt();
	
	std::cout<<"Default count="<<count[0]<<"\n";
	gpu_vector2d<float> arr(w,h);
	arr=fillArray();
	
	gpu_vector2d<vec2> bar(w,h);
	bar=writeArray(w,bar,count);
	std::cout<<"Finished count="<<count[0]<<"\n";
	
	//bar=fillArray(); <- typecheck win!
	//bar=writeArray(w,arr); <- typecheck win!
	
	std::cout<<"result at "<<w<<"x"<<h<<":\n";
	for (int j=0;j<h;j++) {
		for (int i=0;i<w;i++)
			std::cout<<bar(i,j).x<<" ";
		std::cout<<"\n";
	}
	
	return 0;
}
