/**
Bit interleaved array indexing test for EPGPU OpenCL library.
Converts (i,j) first to a Morton and then to a Peano-Hilbert curve.

Dr. Orion Lawlor, lawlor@alaska.edu, 2011-05-16 (Public Domain)
*/
#include <iostream>
#include <stdlib.h>
#include "epgpu.h" /* Orion Lawlor's Expressive Programming for GPU library */
#include "epgpu.cpp" /* <- just compile it here, now.  No compiler/DLL hassles! */

int main(int argc,char *argv[]) {
	int w,h; w=h=16;
	if (argc>2) {w=atoi(argv[1]); h=atoi(argv[2]); }
	
	static GPU_KERNEL_2D(writeArray, (int w, __global<int *> dest),
		int morton=interleave2D(i,j);
/*
Basic observation:
   Morton "Z" turns into Peano-Hilbert "U" by flipping the top X around:
       new x low bit = old x low bit ^ old y low bit
   
   U can be flipped backwards by flipping the major axis (X or Y).
   U can be rotated 90 degrees

X'_L = X_L ^ X_H ^ Y_L
X'_H = X_H ^ Y_H
Y'_L = Y_L ^ Y_H
Y'_H = Y_H
*/
		int xLm=0x11111111;
		int yLm=0x22222222;
		int xHm=0x44444444;
		int yHm=0x88888888;
		int xL=(morton&xLm)>>0;
		int yL=(morton&yLm)>>1;
		int xH=(morton&xHm)>>2;
		int yH=(morton&yHm)>>3;
		
		int nxL=xL^xH^yL;
		int nxH=xH^yH;
		int nyL=yL^yH;
		int nyH=yH;
		
		int peano=(nyH<<3)|(nxH<<2)|(nyL<<1)|(nxL);
		dest[i+j*w]=peano;
	)
	
	gpu_vector2d<int> bar(w,h);
	bar=writeArray(w,bar);
	
	std::cout<<"result at "<<w<<"x"<<h<<":\n";
	for (int j=0;j<h;j++) {
		for (int i=0;i<w;i++)
			printf("%02x ",bar(i,j));
		printf("\n");
	}
	
	return 0;
}
