/**

*/
#ifndef __EPGPU_INDEXLIST_H
#define __EPGPU_INDEXLIST_H

/** The CPU-side analysis indices we use in this file.  Usually 4 bytes is enough. */
typedef int index_t;

/** An index generator: given an ijk array index, generates a series of
   load or store indices.  Not clear you want the virtual versions here;
   templating on the exact type would typically be more efficient. */
class index_generator {
public:
	/** Return the code name of this index generator */
	virtual const char *get_name(void) const =0;

	/** Return the number of load/store indices we return. */
	virtual int get_index_count(void) const =0;
	
	/** Write our load/store indices into this array. */
	virtual void get_indexes(index_t *store,int i,int j=0,int k=0) const =0;
	
	/** Return compile-ready code for this OpenCL index statement */
	virtual const char *get_index_code(int num) const =0;
	
	/** Return the code name for this index number */
	virtual const char *get_index_name(int num) const =0;
	
	virtual ~index_generator(); // purely to avoid compiler warnings.
};

#define GPU_INDEX_SET(name, constants, initializer, n_indexes_, list_of_indexes) \
class name : public index_generator { \
public: \
	constants; \
	name initializer \
	enum {n_indexes = n_indexes_}; \
	inline const char *get_name(void) const {return #name;} \
	inline int get_index_count(void) const {return n_indexes;} \
	inline void get_indexes(index_t *store,int i,int j=0,int k=0) const { \
		list_of_indexes(MAKEINDEX_STORE) \
  	} \
	inline const char *get_index_code(int num) const {\
		switch (num) { \
		list_of_indexes(MAKEINDEX_CASECODE) \
		default: return 0; \
		}; \
	} \
	inline const char *get_index_name(int num) const {\
		switch (num) { \
		list_of_indexes(MAKEINDEX_CASENAME) \
		default: return 0; \
		}; \
	} \
};

#define MAKEINDEX_STORE(num,name,code) store[num]=(code)
#define MAKEINDEX_CASECODE(num,name,code) case num: return #code
#define MAKEINDEX_CASENAME(num,name,code) case num: return #name




#endif
