/**
Tiny example for EPGPU library.

Dr. Orion Lawlor, lawlor@alaska.edu, 2011-04-27 (Public Domain)
*/
#include <fstream>
#include "epgpu.h" /* Orion Lawlor's Expressive Programming for GPU library */
#include "epgpu.cpp" /* <- just compile it here, now.  No compiler/DLL hassles! */

/* OpenCL code */
GPU_FILLKERNEL_2D(unsigned char,
 mandelbrot,(float sz,float xoff,float yoff),
	/* Create complex numbers c and z */
	float2 c=(float2)(
		i*sz+xoff,
		(h-1-j)*sz+yoff
	);
	float2 z=c;
	/* Run the mandelbrot iteration */
	int count;
	enum { max_count=250};
	for (count=0;count<max_count;count++) {
		if ((z.x*z.x+z.y*z.y)>4.0f) break;
		z=(float2)(
			z.x*z.x-z.y*z.y + c.x,
			2.0f*z.x*z.y + c.y
		);
	}
	/* Return the output pixel color */
	result=count;
)

/* C++ main function */
int main() {
	int w=1024, h=1024;
	gpu_vector2d<unsigned char> img(w,h);
	img=mandelbrot(0.00001,0.317,0.414);
	
	// Write rendered data to output file
	std::ofstream file("mandel.ppm");
	file<<"P5\n"<<w<<" "<<h<<"\n255\n";
	unsigned char *ca=new unsigned char[w*h];
	img.read(ca);
	file.write((char *)ca,w*h);
}
