/**
Tiny example for EPGPU library.

Dr. Orion Lawlor, lawlor@alaska.edu, 2011-04-27 (Public Domain)
*/
#include <fstream>
#include "epgpu.h" /* Orion Lawlor's Expressive Programming for GPU library */
#include "epgpu.cpp" /* <- just compile it here, now.  No compiler/DLL hassles! */

/* Set up initial conditions: basically a big circle */
GPU_FILLKERNEL_2D(float,
stencil_initial,(float sz,float cx,float cy,float r),
	float x=i*sz-cx; float y=j*sz-cy;
	if (fmod(sqrt(x*x+y*y),r)<=0.5*r) {
		result=1.0;
	} else {
		result=0.0;
	}
)

/* Do one neighborhood averaging pass over src vector. */
GPU_FILLKERNEL_2D(float,
stencil_sweep,(__global<float *> src),
	int n=i+w*j; // 2D to 1D indexing
	if (i>0 && i<w-1 && j>0 && j<h-1) { // Interior
		result = (src[n-1]+src[n+1]+
		 		  src[n-w]+src[n+w])*0.25;
	} else { // Boundary--copy old value 
		result = src[n];
	}
)

int main() {
	int w=1024, h=1024;

	gpu_vector2d<float> stencil_src(w,h); 
	stencil_src=stencil_initial(0.01,6.0,2.4,3.0);
	for (int time=0;time<1000;time++) {
	 	gpu_vector2d<float> stencil_dst(w,h);
	 	stencil_dst=stencil_sweep(stencil_src);
	 	std::swap(stencil_dst,stencil_src); /* <- ping-pongs the buffers */
	}
	
	// Write data to PPM output image
	std::ofstream file("stencil_array.ppm",std::ios_base::binary);
	file<<"P5\n"<<w<<" "<<h<<"\n255\n";
	float *fa=new float[w*h]; stencil_src.read(fa);
	char *ca=new char[w*h];
	for (int j=0;j<h;j++)
	   for (int i=0;i<w;i++)
		  ca[i+j*w]=(unsigned char)(255.0*fa[i+j*w]);
	file.write(ca,w*h);
	delete[] fa; delete[] ca;
	
	return 0;
}
