/**
Benchmarking for OpenCL EPGPU library.

Dr. Orion Lawlor, lawlor@alaska.edu, 2011-04-27 (Public Domain)
*/
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <stdlib.h>
#include "epgpu.h" /* Orion Lawlor's Expressive Programming for GPU library */
#include "epgpu.cpp" /* <- just compile it here, now.  No compiler/DLL hassles! */

#include "timingCL.h"

/****************** OpenCL basics ****************/

/* Dimensions for all 2D examples below: */
int w=1024, h=1024;


gpu_array2d<float> transpose_src(h,w),transpose_dst(w,h);

/** Naive transpose */
GPU_FILLKERNEL_2D(float,
 naive_transposeGPU,(__global<float *> src),
   result=src[j+i*h];
);
int naive_transpose(void) {
	transpose_dst=naive_transposeGPU(transpose_src);
	return 0;
}

/**
  Transpose 2D src h x w into 2D dst w x h.
      dst[i+j*w] = src[j+i*h];
  To avoid global memory scatter, does transposes to local memory.
*/
GPU_KERNEL_2D(local_transposeGPU,
 (__global<float *> dst,__global<float *> src,
  int w,int h),
   enum {N=16}; // size of local work groups
   int2 G=N*(int2)(get_group_id(0),get_group_id(1));
   int2 L=  (int2)(get_local_id(0),get_local_id(1));
   __local float loc[N*N]; // transpose here
   
   int2 S=G+(int2)(L.y,L.x); // Read backwards
   if (S.x<w && S.y<h) 
     loc[L.y+N*L.x]=src[S.y+S.x*h];
   
   barrier(CLK_LOCAL_MEM_FENCE);
   
   int2 D=G+L; // Write forwards
   if (D.x<w && D.y<h) 
     dst[D.x+D.y*w]=loc[L.x+N*L.y];
);

/** Bit-interleave (Morton 2D order) */
GPU_KERNEL(
transpose2DinterK, (__global<float *> dst,__global<float *> src), 
	int j=get_global_id(1);
	dst[interleave2D(j,i)]=src[interleave2D(i,j)];
)


int transpose2Dinter(void) { 
	transpose2DinterK(transpose_dst,transpose_src).run(w,h);
	return 0; 
}

int local_transpose(void) {
	local_transposeGPU.override_local[0]=16;
	local_transposeGPU.override_local[1]=16;
	local_transposeGPU(transpose_dst,transpose_src,w,h).run(w,h);
	return 0;
}



